package com.tiptimes.ctrl;
import com.tiptimes.model.Answer;
import com.tiptimes.model.File;
import com.tiptimes.model.Questionnaire;
import com.tiptimes.model.Record;
import com.tiptimes.model.Review;
import com.tiptimes.service.FileService;
import com.tiptimes.service.QuestionnaireService;
import com.tiptimes.util.CompressUtils;
import com.tiptimes.util.FileUtil;
import com.tiptimes.util.UUIDUtils;
import org.apache.commons.io.FileUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

@Controller
@RequestMapping({"/questionnaire"})
public class QuestionnaireCtrl {
    @Autowired
    private QuestionnaireService questionnaireService;

    @Autowired
    private FileService fileServer;

    private static String fileUrl = "";

    public QuestionnaireCtrl() {
    }

    /**
     * 数据上报
     * @param questionnaire 上报数据
     * @return
     */
    @RequestMapping({"/save"})
    @ResponseBody
    public Map<String, Object> save(@RequestBody Questionnaire questionnaire, HttpServletRequest request){
        int result;
        //1.通过上报人和问卷id查询上报记录id
        Long recordId=this.questionnaireService.selectRecordId(questionnaire.getId(),
                request.getSession().getAttribute("realID").toString());
        //2.若recordId为空去添加上报记录，并返回自增id
        if(recordId==null) {
            Record record = new Record(questionnaire.getId(),questionnaire.getQuestionnaireName(),
                    0,questionnaire.getWritePeople(),questionnaire.getPhoneNumber(),questionnaire.getCreateName(),
                    new Date(),questionnaire.getCreateId());
            recordId=this.questionnaireService.insertRecord(record);
        }
        //3.根据上报记录id删除原上报数据
            this.questionnaireService.delAnswer(recordId);
        //4.新增新的上报数据
            List<Answer> answerList=questionnaire.getAnswersList();
            if(answerList.size()>0) {
                result = this.questionnaireService.insertAnswer(answerList, recordId);
            } else if(answerList.size()==0&&questionnaire.getType()==2) {
                result=1;
            } else {
                Map<String, Object> t = new HashMap();
                t.put("result", false);
                t.put("msg", "上传数据不能为空");
                return t;
            }
        //5.根据类型判断是否修改上报状态
        if(questionnaire.getType()==1) {//上报
            //修改数据上报状态
            Record r = new Record();
            r.setId(recordId);
            r.setProcessStatus(1);
            result=this.questionnaireService.updateRecord(r);
        }
        Map<String, Object> map = new HashMap();
        if(result>0) {
            map.put("result", true);
            map.put("msg", "操作成功");
        } else {
            map.put("result", false);
            map.put("msg", "操作失败");
        }
            return map;
        }

    /**
     * 审核页面操作
     * @param record 驳回
     * @return
     */
    @RequestMapping({"/checkReject"})
    @ResponseBody
    public Map<String, Object> checkReject(@RequestBody Record record){
        if(record.getReviews()!=null) {
            //1.通过记录id删除原有备注
            this.questionnaireService.delView(record.getReviews().get(0).getRecordId());
            //2.添加新的备注
            this.questionnaireService.insertReview(record.getReviews(), record.getReviews().get(0).getRecordId());
        }
        int result=this.questionnaireService.updateRecord(record);
        Map<String, Object> map = new HashMap();
        if(result>0) {
            map.put("result", true);
            map.put("msg", "操作成功");
        } else {
            map.put("result", false);
            map.put("msg", "操作失败");
        }
        return map;
    }

    /**
     * 根据上报id查看上报数据详情
     * @param questionnaireUuid 问卷
     * @return
     */
    @RequestMapping({"/selectDetails"})
    @ResponseBody
    public Map<String, Object> selectDetails(Long questionnaireUuid, HttpServletRequest request){
        Long recordId=this.questionnaireService.selectRecordId(questionnaireUuid,
                request.getSession().getAttribute("realID").toString());
        //1.根据记录id查询上报数据详情
        List<Answer>answerList=this.questionnaireService.selectAnswerByRecordId(recordId);
        //2.根据记录id查询审核备注详情
        List<Review> reviews=this.questionnaireService.selectReviewByRecordId(recordId);
        Map<String, Object> map = new HashMap();
        map.put("result", true);
        map.put("msg", "");
        Map<String, Object> data = new HashMap();
        data.put("answerList",answerList);
        if(reviews.size()>0) {
            data.put("reviews", reviews);
        }
        //3.根据记录id查询状态
        Record record= this.questionnaireService.selectById(recordId);
        data.put("createBy",record.getCreateBy());
        data.put("phoneNumber",record.getPhoneNumber());
        if(record.getProcessStatus()==2) {
            data.put("reason", record.getReason());
        }
        map.put("data",data);
        return map;
    }

    /**
     * 保存备注
     * @param review 备注
     * @return
     */
    @RequestMapping({"/review"})
    @ResponseBody
    public Map<String, Object> review(@RequestBody List<Review> review){
        Map<String, Object> map = new HashMap();
        if(review==null||review.size()==0) {
            map.put("result", false);
            map.put("msg", "不存在需要保存的备注");
            return map;
        }
        int result;
        //1.通过记录id删除原有备注
        this.questionnaireService.delView(review.get(0).getRecordId());
        //2.添加新的备注
        result=this.questionnaireService.insertReview(review,review.get(0).getRecordId());
        if(result>0) {
            map.put("result", true);
            map.put("msg", "操作成功");
        } else {
            map.put("result", false);
            map.put("msg", "操作失败");
        }
        return map;
    }



    /**
     * 按条件查询上报记录
     * @param record 筛选条件
     * @return
     */
    @RequestMapping({"/selectRecordList"})
    @ResponseBody
    public Map<String, Object> selectRecordList(Record record){
        List<Record> recordList= this.questionnaireService.selectRecordList(record);
        Map<String, Object> map = new HashMap();
        Map<String, Object> data = new HashMap();
        map.put("result", true);
        map.put("msg", "");
        data.put("recordList",recordList);
        map.put("data",data);
        return map;
    }

    @RequestMapping({"downZipByFileId"})
    public String downZipByFileId(Long id, HttpServletRequest request, HttpServletResponse response) throws IOException {
        String forwardName = "/user/file";
        //1.根据选项查询文件列表
        String forderName = UUIDUtils.getUUID();
        String tempPath = this.getfileUrl();
        String zipName="上传文件";
        String fileName;
        String myPath = tempPath + forderName + "/" + zipName + "/";
        java.io.File fileForExists = new java.io.File(myPath);
        if (!fileForExists.exists()) {
            FileUtil.mkdirFolders(myPath);
        }
        List<String> fileID = this.fileServer.selectFileId(id);
        for(int i=0;i<fileID.size();++i) {
            File file = this.fileServer.selectFileByFileID(fileID.get(i));
            try {
                if (file != null) {
                    String fileNum = file.getFileNum();
                    fileName = file.getFileName();
                    String originPath = "";
                    if (fileNum.startsWith("http")) {
                        originPath = fileNum;
                    } else {
                        originPath = this.getfileUrl() + fileNum;
                    }
                    String targetPath = myPath + fileName;
                    StringBuffer fileNameForRepeat = new StringBuffer();
                    fileNameForRepeat.append(fileName.substring(0, fileName.lastIndexOf(".")));
                    fileNameForRepeat.append("_" + i);
                    fileNameForRepeat.append(fileName.substring(fileName.lastIndexOf(".")));
                    String targetPathForRepeat = myPath + fileNameForRepeat;
                    FileUtil.copyFileByPath(originPath, targetPath, targetPathForRepeat);
                }
            } catch (Exception var22) {
                System.out.println(var22);
            }
        }

        String zipPath = tempPath + forderName;
        java.io.File zipFile = new java.io.File(zipPath);
        zipName = forderName + ".zip";
        (new CompressUtils(new java.io.File(tempPath, zipName))).zipFiles(zipFile);
        FileUtils.deleteDirectory(new java.io.File(zipPath));
        try {
            String zipPathExist = tempPath + zipName;
            java.io.File zipFileDownLoad = new java.io.File(zipPathExist);
            if (zipFileDownLoad.exists()) {
                FileInputStream in = new FileInputStream(zipFileDownLoad);
                ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
                byte[] temp = new byte[1024];
                boolean var33 = false;
                int size;
                while((size = in.read(temp)) != -1) {
                    out.write(temp, 0, size);
                }
                in.close();
                byte[] data = out.toByteArray();
                out.close();
                fileName = new String(zipName.getBytes(), "ISO8859-1");
                response.setHeader("Content-Disposition", "attachment; filename=\"" + fileName + "\"");
                response.addHeader("Content-Length", "" + data.length);
                response.setContentType("application/x-zip-compressed;charset=ISO8859-1");
                OutputStream outputStream = response.getOutputStream();
                outputStream.write(data);
                outputStream.flush();
                outputStream.close();
                zipFileDownLoad.delete();
            } else {
                System.out.println("下载的文件不存在");
            }
        } catch (Exception var23) {
            var23.printStackTrace();
        }
        return null;
    }

    private String getfileUrl() throws IOException {
        if ("".equals(fileUrl)) {
            Properties props = new Properties();
            InputStream inputStream = this.getClass().getClassLoader().getResourceAsStream("file.properties");
            props.load(inputStream);
            System.out.println(props.getProperty("fileUrl"));
            fileUrl = props.getProperty("fileUrl");
            java.io.File tempFile = new java.io.File(fileUrl);
            if (!tempFile.exists()) {
                tempFile.mkdirs();
            }

            inputStream.close();
        }

        return fileUrl;
    }
}

